@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.1 (the "License").  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON- INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -124,6 +123,7 @@ char *dyld_image_suffix = NULL;
 char *dyld_insert_libraries = NULL;
 
 enum bool dyld_print_libraries = FALSE;
+static enum bool dyld_print_libraries_post_launch = FALSE;
 enum bool dyld_trace = FALSE;
 enum bool dyld_mem_protect = FALSE;
 enum bool dyld_ebadexec_only = FALSE;
@@ -153,6 +153,24 @@ enum bool profile_server = FALSE;
 enum bool prebinding = TRUE;
 enum bool launched = FALSE;
 enum bool executable_prebound = FALSE;
+/*
+ * This is TRUE when all libraries currently loaded are two-level, prebound,
+ * and and all of their modules are bound.  It will get set to TRUE in 
+ * set_all_twolevel_modules_prebound() right before the program is launched.
+ * It may then be cleared if a library is later dynamically loaded that is
+ * not two-level, prebound and all modules bound.
+ */
+enum bool all_twolevel_modules_prebound = FALSE;
+
+/*
+ * When loading libraries after the program is launched if we will try to use
+ * the prebinding if all the previously loaded libraries were two-level and all
+ * the modules were prebound if the library being loaded is also prebound and
+ * two-level.  When that is happening trying_to_use_prebinding_post_launch is
+ * set to true.  And the prebinding for the libraries is not touched until
+ * all the dependent libraries are loaded. Once they are loaded then ...
+ */
+enum bool trying_to_use_prebinding_post_launch = FALSE;
 
 static struct segment_command *data_seg;
 #if 0 /* disable until NXProtectZone() works again */
@@ -231,7 +249,8 @@ char **envp)
 #if defined(__GONZO_BUNSEN_BEAKER__) && defined(__ppc__)
 	if(host_basic_info.cpu_type == CPU_TYPE_POWERPC &&
 	   (host_basic_info.cpu_subtype == CPU_SUBTYPE_POWERPC_7400 ||
-	    host_basic_info.cpu_subtype == CPU_SUBTYPE_POWERPC_7450))
+	    host_basic_info.cpu_subtype == CPU_SUBTYPE_POWERPC_7450 ||
+	    host_basic_info.cpu_subtype == CPU_SUBTYPE_POWERPC_970))
 	    processor_has_vec = TRUE;
 #endif
 
@@ -328,7 +347,7 @@ char **envp)
 	}
 	if(prebinding == FALSE){
 	    /*
-	     * The program was not fully prebound but it we are not forcing
+	     * The program was not fully prebound but if we are not forcing
 	     * flat namespace semantics we can still use any sub trees of
 	     * libraries that are all two-level namespace and prebound.
 	     */
@@ -338,7 +357,7 @@ char **envp)
 	    /*
 	     * First undo any images that were prebound.
 	     */
-	    undo_prebound_images();
+	    undo_prebound_images(FALSE);
 
 	    /*
 	     * Build the initial list of non-lazy symbol references based on the
@@ -351,7 +370,7 @@ char **envp)
 	    /*
 	     * With the undefined list set up link in the needed modules.
 	     */
-	    link_in_need_modules(FALSE, FALSE);
+	    link_in_need_modules(FALSE, FALSE, NULL);
 	}
 	else{
 	    if(dyld_prebind_debug != 0){
@@ -362,6 +381,12 @@ char **envp)
 		    print("dyld: %s: prebinding enabled\n", argv[0]);
 	    }
 	}
+	/*
+	 * Now with the program about to be launched set
+	 * all_twolevel_modules_prebound to TRUE if all libraries are two-level,
+	 * prebound and all modules in them are linked.
+	 */
+	set_all_twolevel_modules_prebound();
 	launched = TRUE;
 
 	/*
@@ -373,6 +398,9 @@ char **envp)
 		"program not started)", argv[0]);
 	    link_edit_error(DYLD_FILE_ACCESS, EBADEXEC, argv[0]);
 	}
+	
+	if(dyld_print_libraries_post_launch == TRUE)
+	    dyld_print_libraries = TRUE;
 
 	/* release lock for dyld data structures */
 	release_lock();
@@ -519,6 +547,10 @@ char *envp[])
 		                sizeof("DYLD_PRINT_LIBRARIES=") - 1) == 0){
 		    dyld_print_libraries = TRUE;
 		}
+		else if(strncmp(*p, "DYLD_PRINT_LIBRARIES_POST_LAUNCH=",
+			sizeof("DYLD_PRINT_LIBRARIES_POST_LAUNCH=") - 1) == 0){
+		    dyld_print_libraries_post_launch = TRUE;
+		}
 		else if(strncmp(*p, "DYLD_TRACE=",
 		                sizeof("DYLD_TRACE=") - 1) == 0){
 		    dyld_trace = TRUE;
@@ -733,3 +765,39 @@ void)
 	    link_edit_error(DYLD_MACH_RESOURCE, r, "dyld");
 	}
 }
+
+/*
+ * To avoid linking in libm.  These variables are defined as they are used in
+ * pthread_init() to put in place a fast sqrt().
+ */
+size_t hw_sqrt_len = 0;
+
+double
+sqrt(double x)
+{
+	return(0.0);
+}
+double
+hw_sqrt(double x)
+{
+	return(0.0);
+}
+
+/*
+ * More stubs to avoid linking in libm.  This works as along as we don't use
+ * long doubles.
+ */
+#ifdef __ppc__
+long
+__fpclassifyd(double x) /* ppc doesn't support long doubles */
+{
+	return(0);
+}
+#endif /* __ppc__ */
+#ifdef __i386__
+long
+__fpclassify(long double x)
+{
+	return(0);
+}
+#endif /* __i386__ */
