@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -24,7 +22,7 @@
  */
 #ifdef SHLIB
 #include "shlib.h"
-#endif SHLIB
+#endif /* SHLIB */
 /*
  * This file contains the routines to do relocation for hp pa-risc.
  */
@@ -101,7 +99,7 @@ struct section_map *section_map)
 	pair_r_symbolnum = 0;
 	pair_r_value = 0;
 	pair_local_map = NULL;
-#endif defined(DEBUG) || defined(RLD)
+#endif /* defined(DEBUG) || defined(RLD) */
 
 	for(i = 0; i < section_map->s->nreloc; i++){
 	    force_extern_reloc = FALSE;
@@ -385,13 +383,13 @@ struct section_map *section_map)
 		/*
 		 * If the symbol is undefined (or common) or a global coalesced 
 		 * symbol where we need to force an external relocation entry
-		 * and we are not prebinding no relocation is done.
-		 * Or if the output file is MH_DYLIB no relocation is done
+		 * and we are not prebinding no relocation is done.  Or if the
+		 * output file is a multi module MH_DYLIB no relocation is done
 		 * unless the symbol is a private extern or we are prebinding.
 		 */
 		if(((merged_symbol->nlist.n_type & N_TYPE) == N_UNDF) ||
 		   (force_extern_reloc == TRUE && prebinding == FALSE) ||
-		   (filetype == MH_DYLIB &&
+		   ((filetype == MH_DYLIB && multi_module_dylib == TRUE) &&
 		    (((merged_symbol->nlist.n_type & N_PEXT) != N_PEXT) &&
 		     prebinding == FALSE) ) )
 		    value = 0;
@@ -808,7 +806,9 @@ struct section_map *section_map)
 						    section_map, r_address) +
 					     8;
 				if(save_reloc == 0 &&
-				   (filetype != MH_DYLIB || (r_extern == 1 &&
+				   ((filetype != MH_DYLIB ||
+			             multi_module_dylib == FALSE) ||
+				    (r_extern == 1 &&
 				    (merged_symbol->nlist.n_type & N_PEXT) ==
 								N_PEXT)) &&
 				   (output_for_dyld == FALSE || r_extern == 0 ||
@@ -1002,7 +1002,8 @@ struct section_map *section_map)
 							     r_address) +
 				    8);
 		    if(save_reloc == 0 &&
-		       (filetype != MH_DYLIB || (r_extern == 1 &&
+		       ((filetype != MH_DYLIB || multi_module_dylib == FALSE) ||
+			(r_extern == 1 &&
 			(merged_symbol->nlist.n_type & N_PEXT) == N_PEXT)) &&
 		       (output_for_dyld == FALSE || r_extern == 0 ||
 			(merged_symbol->nlist.n_type & N_TYPE) != N_UNDF) &&
@@ -1055,18 +1056,19 @@ update_reloc:
 		     * For external relocation entries that the symbol is
 		     * defined (not undefined or common) but not when we are
 		     * forcing an external relocation entry for a global
-		     * coalesced symbol and if the output file is not MH_DYLIB
-		     * or the symbol is a private extern it is changed to a
-		     * local relocation entry using the section that symbol is
-		     * defined in.  If still undefined or forcing an external
-		     * relocation entry for a global coalesced symbol then the
-		     * index of the symbol in the output file is set into
-		     * r_symbolnum.
+		     * coalesced symbol and if the output file is not a multi
+		     * module MH_DYLIB or the symbol is a private extern, it is
+		     * changed to a local relocation entry using the section
+		     * that symbol is defined in.  If still undefined or forcing
+		     * an external relocation entry for a global coalesced
+		     * symbol, then the index of the symbol in the output file
+		     * is set into r_symbolnum.
 		     */
 		    else if((merged_symbol->nlist.n_type & N_TYPE) != N_UNDF &&
 		            (merged_symbol->nlist.n_type & N_TYPE) != N_PBUD &&
 		            force_extern_reloc == FALSE &&
-		            (filetype != MH_DYLIB ||
+		            ((filetype != MH_DYLIB ||
+			      multi_module_dylib == FALSE) ||
 			     (merged_symbol->nlist.n_type & N_PEXT) == N_PEXT)){
 			reloc->r_extern = 0;
 			/*
@@ -1227,7 +1229,7 @@ update_reloc:
 		    }
 		}
 	    }
-#endif !defined(RLD)
+#endif /* !defined(RLD) */
 	    /*
 	     * If their was a paired relocation entry then it has been processed
 	     * so skip it by incrementing the index of the relocation entry that
