@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.1 (the "License").  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON- INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -287,13 +286,15 @@ char *arch_name)
 {
 	if(ofile->file_type == OFILE_FAT){
 	    if(ofile->arch_type != OFILE_Mach_O ||
-	       ofile->mh->filetype != MH_DYLIB)
+	       (ofile->mh->filetype != MH_DYLIB &&
+	        ofile->mh->filetype != MH_DYLIB_STUB))
 	    fatal("for architecture %s file: %s is not a dynamic shared "
 		  "library", ofile->arch_flag.name, ofile->file_name);
 	}
 	else
 	    if(ofile->file_type != OFILE_Mach_O ||
-	       ofile->mh->filetype != MH_DYLIB)
+	       (ofile->mh->filetype != MH_DYLIB &&
+	        ofile->mh->filetype != MH_DYLIB_STUB))
 		fatal("file: %s is not a dynamic shared library",
 		      ofile->file_name);
 }
@@ -361,7 +362,7 @@ enum bool new_api_allowed)
 	}
 	for(i = 0; i < old_st->nsyms; i++){
 	    if(old_symbols[i].n_un.n_strx != 0 &&
-	       old_symbols[i].n_un.n_strx > old_st->strsize){
+	       (unsigned long)old_symbols[i].n_un.n_strx > old_st->strsize){
 		if(arch_name != NULL)
 		    fatal("malformed dynamic shared library: %s (for "
 			"architecture %s) (bad string table index for symbol "
@@ -421,7 +422,7 @@ enum bool new_api_allowed)
 	}
 	for(i = 0; i < new_st->nsyms; i++){
 	    if(new_symbols[i].n_un.n_strx != 0 &&
-	       new_symbols[i].n_un.n_strx > new_st->strsize){
+	       (unsigned long)new_symbols[i].n_un.n_strx > new_st->strsize){
 		if(arch_name != NULL)
 		    fatal("malformed dynamic shared library: %s (for "
 			"architecture %s) (bad string table index for symbol "
