@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -114,7 +112,8 @@ char **argv,
 char **envp)
 {
     struct flags flags;
-    unsigned long i, j, nfiles;
+    int i;
+    unsigned long j, nfiles;
     char *endp;
     struct arch_flag *arch_flags;
     unsigned long narch_flags;
